@@ -697,16 +697,16 @@ pub async fn query<D: DeserializeOwned, P: Into<PartitionKeys>, C: ToString>(
     query_crosspartition(collection_name, pk, query, max_count, false).await
 }
 
-/// Uploads a new form data image to the blob storage and returns the image_id
-pub async fn upload_image(mut f: FormData) -> Result<String, CosmosError> {
+
+/// Uploads form data to the blob storage and returns the blob id
+pub async fn upload_blob(mut f: FormData, blob_type: &str, expected_content_type: &str, storage_container: &str) -> Result<String, CosmosError> {
     while let Some(r) = f.next().await {
         match r {
             Ok(part) => {
-                if part.name() == "image" {
+                if part.name() == blob_type {
                     if let Some(g) = part.content_type() {
                         let content_type = String::from(g);
-
-                        if content_type.starts_with("image/") {
+                        if content_type.starts_with(&expected_content_type) {
                             match part.filename() {
                                 Some(n) => {
                                     let pos = n.find('.');
@@ -714,7 +714,7 @@ pub async fn upload_image(mut f: FormData) -> Result<String, CosmosError> {
                                         let ext = String::from(n);
                                         let ext = &ext[pos..];
 
-                                        //FIXME: Here we are using a Vec<u8> when we probably want to
+                                        //FIXME(Jonathan): Here we are using a Vec<u8> when we probably want to
                                         //use a bytes::Bytes. Additionally we are copying the buffer
                                         //when moving or referencing is prerferrable.
                                         //The reason we're doing it this way is because there are
@@ -728,20 +728,20 @@ pub async fn upload_image(mut f: FormData) -> Result<String, CosmosError> {
                                                 }
                                                 Err(err) => {
                                                     return Err(new_cosmos_error_kind(
-                                                        format!(
-                                                            "Error getting image data: {:?}",
-                                                            err
-                                                        ),
-                                                        CosmosErrorKind::BlobError,
+                                                            format!(
+                                                                "Error getting {} data: {:?}",
+                                                                blob_type, err
+                                                            ),
+                                                            CosmosErrorKind::BlobError,
                                                     ));
                                                 }
                                             }
                                         }
 
-                                        let mut image_id = Uuid::new_v4().to_string();
+                                        let mut blob_id = Uuid::new_v4().to_string();
 
-                                        // Add extension to image id.
-                                        image_id.push_str(ext);
+                                        // Add extension to blob id.
+                                        blob_id.push_str(ext);
                                         let http_client: Arc<Box<dyn HttpClient>> =
                                             Arc::new(Box::new(reqwest::Client::new()));
 
@@ -750,9 +750,9 @@ pub async fn upload_image(mut f: FormData) -> Result<String, CosmosError> {
                                             STORAGE_ACCOUNT.to_string(),
                                             STORAGE_MASTER_KEY.to_string(),
                                         )
-                                        .as_storage_client()
-                                        .as_container_client(&*IMAGES_STORAGE_CONTAINER)
-                                        .as_blob_client(&image_id);
+                                            .as_storage_client()
+                                            .as_container_client(storage_container)
+                                            .as_blob_client(&blob_id);
 
                                         // Helps preventing spurious data to be uploaded.
                                         let digest = md5::compute(&buf[..]).into();
@@ -763,36 +763,36 @@ pub async fn upload_image(mut f: FormData) -> Result<String, CosmosError> {
                                                 .hash(&digest)
                                                 .execute()
                                                 .await
-                                            {
-                                                Ok(r) => r,
-                                                Err(err) => {
-                                                    return Err(new_cosmos_error_kind(
-                                                                format!("Could not add image to storage account {:?}", err),
+                                                {
+                                                    Ok(r) => r,
+                                                    Err(err) => {
+                                                        return Err(new_cosmos_error_kind(
+                                                                format!("Could not add blob to storage account {:?}", err),
                                                                 CosmosErrorKind::BlobError));
-                                                }
-                                            };
+                                                    }
+                                                };
                                         }
 
-                                        // Set image id.
-                                        return Ok(image_id);
+                                        // Set blob id.
+                                        return Ok(blob_id);
                                     } else {
                                         return Err(new_cosmos_error_kind(
-                                            "Could not get filename for image.",
-                                            CosmosErrorKind::BlobError,
+                                                "Could not get filename for blob.",
+                                                CosmosErrorKind::BlobError,
                                         ));
                                     }
                                 }
                                 None => {
                                     return Err(new_cosmos_error_kind(
-                                        "Could not get filename for image.",
-                                        CosmosErrorKind::BlobError,
+                                            "Could not get filename for blob.",
+                                            CosmosErrorKind::BlobError,
                                     ));
                                 }
                             }
                         } else {
                             return Err(new_cosmos_error_kind(
-                                "Image does not have an image content-type.",
-                                CosmosErrorKind::BlobError,
+                                    format!("Blob does not have a {} content-type.", expected_content_type),
+                                    CosmosErrorKind::BlobError,
                             ));
                         }
                     }
@@ -800,131 +800,26 @@ pub async fn upload_image(mut f: FormData) -> Result<String, CosmosError> {
             }
             Err(err) => {
                 return Err(new_cosmos_error_kind(
-                    format!("Error getting multipart data {:?}", err),
-                    CosmosErrorKind::BlobError,
+                        format!("Error getting multipart data {:?}", err),
+                        CosmosErrorKind::BlobError,
                 ));
             }
         }
     }
     return Err(new_cosmos_error_kind(
-        "No image provided",
-        CosmosErrorKind::BlobError,
+            "No blob provided",
+            CosmosErrorKind::BlobError,
     ));
 }
 
+/// Uploads a new form data image to the blob storage and returns the image_id
+pub async fn upload_image(f: FormData) -> Result<String, CosmosError> {
+    upload_blob(f, "image", "image", &*IMAGES_STORAGE_CONTAINER).await
+}
+
 /// Uploads a new form data video to the blob storage and returns the video_id
-pub async fn upload_video(mut f: FormData) -> Result<String, CosmosError> {
-    while let Some(r) = f.next().await {
-        match r {
-            Ok(part) => {
-                if part.name() == "video" {
-                    if let Some(g) = part.content_type() {
-                        let content_type = String::from(g);
-
-                        if content_type.starts_with("video/") {
-                            match part.filename() {
-                                Some(n) => {
-                                    let pos = n.find('.');
-                                    if let Some(pos) = pos {
-                                        let ext = String::from(n);
-                                        let ext = &ext[pos..];
-
-                                        //FIXME: Here we are using a Vec<u8> when we probably want to
-                                        //use a bytes::Bytes. Additionally we are copying the buffer
-                                        //when moving or referencing is prerferrable.
-                                        //The reason we're doing it this way is because there are
-                                        //currently some versioning issues with azure
-                                        let mut buf: Vec<u8> = Vec::with_capacity(64);
-                                        let mut s = part.stream();
-                                        while let Some(r) = s.next().await {
-                                            match r {
-                                                Ok(mut b) => {
-                                                    buf.extend(b.copy_to_bytes(b.remaining()));
-                                                }
-                                                Err(err) => {
-                                                    return Err(new_cosmos_error_kind(
-                                                        format!(
-                                                            "Error getting video data: {:?}",
-                                                            err
-                                                        ),
-                                                        CosmosErrorKind::BlobError,
-                                                    ));
-                                                }
-                                            }
-                                        }
-
-                                        let mut image_id = Uuid::new_v4().to_string();
-
-                                        // Add extension to image id.
-                                        image_id.push_str(ext);
-                                        let http_client: Arc<Box<dyn HttpClient>> =
-                                            Arc::new(Box::new(reqwest::Client::new()));
-
-                                        let blob_client = StorageAccountClient::new_access_key(
-                                            http_client.clone(),
-                                            STORAGE_ACCOUNT.to_string(),
-                                            STORAGE_MASTER_KEY.to_string(),
-                                        )
-                                        .as_storage_client()
-                                        .as_container_client(&*VIDEOS_STORAGE_CONTAINER)
-                                        .as_blob_client(&image_id);
-
-                                        // Helps preventing spurious data to be uploaded.
-                                        let digest = md5::compute(&buf[..]).into();
-                                        {
-                                            match blob_client
-                                                .put_block_blob(buf)
-                                                .content_type(content_type.as_str())
-                                                .hash(&digest)
-                                                .execute()
-                                                .await
-                                            {
-                                                Ok(r) => r,
-                                                Err(err) => {
-                                                    return Err(new_cosmos_error_kind(
-                                                                format!("Could not add video to storage account {:?}", err),
-                                                                CosmosErrorKind::BlobError));
-                                                }
-                                            };
-                                        }
-
-                                        // Set image id.
-                                        return Ok(image_id);
-                                    } else {
-                                        return Err(new_cosmos_error_kind(
-                                            "Could not get filename for video.",
-                                            CosmosErrorKind::BlobError,
-                                        ));
-                                    }
-                                }
-                                None => {
-                                    return Err(new_cosmos_error_kind(
-                                        "Could not get filename for video.",
-                                        CosmosErrorKind::BlobError,
-                                    ));
-                                }
-                            }
-                        } else {
-                            return Err(new_cosmos_error_kind(
-                                "Video does not have an video content-type.",
-                                CosmosErrorKind::BlobError,
-                            ));
-                        }
-                    }
-                }
-            }
-            Err(err) => {
-                return Err(new_cosmos_error_kind(
-                    format!("Error getting multipart data {:?}", err),
-                    CosmosErrorKind::BlobError,
-                ));
-            }
-        }
-    }
-    return Err(new_cosmos_error_kind(
-        "No video provided",
-        CosmosErrorKind::BlobError,
-    ));
+pub async fn upload_video(f: FormData) -> Result<String, CosmosError> {
+    upload_blob(f, "video", "video", &*VIDEOS_STORAGE_CONTAINER).await
 }
 
 pub struct CosmosSaga {
